//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "InjTre.h"

ClassImp(InjTre)


/////////////////////////////////////////////////////////////////////////
InjTre::InjTre(const string aInjType, const int aVerbose){ 
/////////////////////////////////////////////////////////////////////////

  Verbose=aVerbose;

  // injection TTree
  InjTree = new TTree(aInjType.c_str(),aInjType.c_str());
  InjTree->Branch("ra",           &inj_ra,    "ra/D");
  InjTree->Branch("dec",          &inj_dec,   "dec/D");
  InjTree->Branch("time",         &inj_time,  "time/D");
  InjTree->Branch("eccentricity", &inj_ecc,   "eccentricity/D");
  InjTree->Branch("polarization", &inj_psi,   "polarization/D");
  InjTree->Branch("amplitude",    &inj_amp,   "amplitude/D");
  InjTree->Branch("sigma",        &inj_sigma, "sigma/D");

  // waveform specific
  if(!aInjType.compare("SineGauss"))
    InjTree->Branch("frequency",  &inj_f0,    "frequency/D");

}

/////////////////////////////////////////////////////////////////////////
InjTre::~InjTre(void){
/////////////////////////////////////////////////////////////////////////
  delete InjTree;
}

/////////////////////////////////////////////////////////////////////////
int InjTre::AddGauss(const double aRa, 
		     const double aDec, 
		     const double aTime, 
		     const double aEccentricity, 
		     const double aPolarization, 
		     const double aAmplitude, 
		     const double aSigma){
/////////////////////////////////////////////////////////////////////////
  if(((string)InjTree->GetTitle()).compare("Gauss")){
    cerr<<"InjTre::AddGauss: the InjTre object was not constructed for Gauss-type injections"<<endl;
    return -1;
  }
  inj_ra=aRa;
  inj_dec=aDec;
  inj_time=aTime;
  inj_ecc=aEccentricity;
  inj_psi=aPolarization;
  inj_amp=aAmplitude;
  inj_sigma=aSigma;
  InjTree->Fill();
  return InjTree->GetEntries();
}

/////////////////////////////////////////////////////////////////////////
int InjTre::AddSineGauss(const double aRa, 
			  const double aDec, 
			  const double aTime, 
			  const double aEccentricity, 
			  const double aPolarization, 
			  const double aAmplitude, 
			  const double aSigma, 
			  const double aFrequency){
/////////////////////////////////////////////////////////////////////////
  if(((string)InjTree->GetTitle()).compare("SineGauss")){
    cerr<<"InjTre::AddGauss: the InjTre object was not constructed for SineGauss-type injections"<<endl;
    return -1;
  }

  inj_ra=aRa;
  inj_dec=aDec;
  inj_time=aTime;
  inj_ecc=aEccentricity;
  inj_psi=aPolarization;
  inj_amp=aAmplitude;
  inj_sigma=aSigma;
  inj_f0=aFrequency;
  InjTree->Fill();
  return InjTree->GetEntries();
}

/////////////////////////////////////////////////////////////////////////
bool InjTre::Write(const string aRootFileName){
/////////////////////////////////////////////////////////////////////////
  TFile finj(aRootFileName.c_str(),"recreate");
  finj.cd();
  InjTree->Write();
  finj.Close();
  return true;
}







