//////////////////////////////////////////////////////////////////////////////
//  Author : florent robinet (LAL - Orsay): robinet@lal.in2p3.fr
//////////////////////////////////////////////////////////////////////////////
#include "ReadTriggers.h"
#include "MakeTriggers.h"

using namespace std;

int main (int argc, char* argv[]){

  // check the argument
  if(argc!=3){
    cerr<<argv[0]<<"usage:"<<endl; 
    cerr<<argv[0]<<" [outdir] [file pattern]"<<endl; 
    cerr<<endl; 
    cerr<<"[outdir]:        output directory where to put merged files"<<endl; 
    cerr<<"[file pattern]:  pattern for ROOT files to merge"<<endl; 
    cerr<<endl; 
    cerr<<" For example:"<<endl; 
    cerr<<argv[0]<<" /path/to/my/outdir \"./file1.root ./file1.root ./all_the_other_files_*.root\""<<endl; 
    cerr<<" this command will produce a file named /path/to/my/outdir/[stream_name]_[gps_start]_[duration].root with all the triggers."<<endl; 
    return 1;
  }

  // outdir
  string outdir = (string)argv[1];
  system(("mkdir -p "+outdir).c_str());
  
  // input files
  string infiles = (string)argv[2];
 
  cout<<"**********************************************************"<<endl;
  cout<<"triggermerge: merging files in \""<<infiles<<"\""<<endl;
  cout<<"**********************************************************"<<endl;

  // extract segments
  cout<<"extracting metadata..."<<endl;
  ReadTriggerMetaData *meta = new ReadTriggerMetaData(infiles,"",0);
  double livetime=meta->GetSegments()->GetLiveTime();
  if(livetime<=0){
    cerr<<"---> no livetime!"<<endl;
    delete meta;
    return 0;
  }

  TTree *segtree = meta->GetSegments()->GetTree();
  cout<<"              "<<meta->GetSegments()->GetNsegments()<<" segments, livetime = "<<livetime<<"s"<<endl;
  
  // stream
  Streams *St = new Streams(meta->GetStreamName(),0);

  // output file name (GWOLLUM convention)
  cout<<"output file name:"<<endl;
  int start=(int)floor(meta->GetSegments()->GetStart(0));
  int duration=(int)ceil(meta->GetSegments()->GetEnd(meta->GetSegments()->GetNsegments()-1))-start;
  ostringstream outfilestream;
  outfilestream<<outdir<<"/"<<St->GetNameConv()<<"_"<<meta->GetProcessName()<<"-"<<start<<"-"<<duration<<".root";
  string outfilename = outfilestream.str();
  cout<<"---> "<<outfilename<<endl;
  delete St;
  
  // extract metadata tree
  cout<<"extracting meta-data..."<<endl;
  TChain *merged_Mtree = new TChain("metadata");
  vector <string> filelist = SplitString(infiles,' ');
  for(int f=0; f<(int)filelist.size(); f++) merged_Mtree->Add(filelist[f].c_str());
  
  // extract trigger tree
  cout<<"extracting triggers..."<<endl;
  TChain *merged_tree = new TChain("triggers");
  for(int f=0; f<(int)filelist.size(); f++) merged_tree->Add(filelist[f].c_str());
  
  TFile* f;

  // there is some triggers
  if(merged_tree->GetEntries()){
    cout<<"writing "<<merged_tree->GetEntries()<<" triggers..."<<endl;
    merged_tree->Merge(outfilename.c_str());
    delete merged_tree;
    if(merged_Mtree->GetNtrees()){
      f = TFile::Open(outfilename.c_str(), "UPDATE");
      f->cd();
      cout<<"writing "<<merged_Mtree->GetEntries()<<" meta-data sets..."<<endl;
      merged_Mtree->Merge(f,0);// NOT SURE ABOUT THE 0 ???
    }
    cout<<"writing "<<meta->GetSegments()->GetNsegments()<<" segments..."<<endl;
    f = TFile::Open(outfilename.c_str(), "UPDATE");
    f->cd();
    segtree->Write();
    f->Close();
  }

  // there is no triggers
  else{
    delete merged_tree;
    cout<<"writing 0 triggers..."<<endl;
    MakeTriggers *Empty = new MakeTriggers(meta->GetStreamName(),0);
    Empty->SetMprocessname(meta->GetProcessName());
    Empty->Append(meta->GetSegments());
    cout<<"writing "<<meta->GetSegments()->GetNsegments()<<" segments..."<<endl;
    if(!Empty->Write(outdir,"root","default","","RECREATE",false).compare("none")){
      cerr<<"---> writing failed"<<endl;
      delete Empty;
      delete merged_Mtree;
      delete meta;
      delete segtree;
      return 1;
    }
    delete Empty;
    if(merged_Mtree->GetNtrees()){
      f = TFile::Open(outfilename.c_str(), "UPDATE");
      f->cd();
      cout<<"writing "<<merged_Mtree->GetEntries()<<" meta-data sets..."<<endl;
      merged_Mtree->Merge(f,0);// NOT SURE ABOUT THE 0 ???
    }

  }

  
  // clean
  delete merged_Mtree;
  delete meta;
  delete segtree;
  cout<<"**********************************************************"<<endl;
  cout<<endl;
  
  return 0;
}

